@@ -14,8 +14,13 @@ import numpy as np
 from azure.mgmt.subscription import SubscriptionClient
 from azure.common.exceptions import CloudError
 
-from ..common.azure_auth import az_connect
-from ..common.azure_auth_core import AzCredentials
+from ..common.azure_auth import (
+    az_connect,
+    AzCredentials,
+    AzureCloudConfig,
+    only_interactive_cred,
+)
+from ..common.cloud_mappings import get_all_endpoints
 
 from ..common.exceptions import (
     MsticpyAzureConfigError,
@@ -109,8 +114,9 @@ class InterfaceItems:
 class AzureData:
     """Class for returning data on an Azure tenant."""
 
-    def __init__(self, connect: bool = False):
+    def __init__(self, connect: bool = False, cloud: str = None):
         """Initialize connector for Azure Python SDK."""
+        self.az_cloud_config = AzureCloudConfig(cloud)
         self.connected = False
         self.credentials: Optional[AzCredentials] = None
         self.sub_client: Optional[SubscriptionClient] = None
@@ -118,26 +124,60 @@ class AzureData:
         self.network_client: Optional[NetworkManagementClient] = None
         self.monitoring_client: Optional[MonitorManagementClient] = None
         self.compute_client: Optional[ComputeManagementClient] = None
+        self.cloud = cloud or AzureCloudConfig().cloud
+        self.endpoints = get_all_endpoints(self.cloud)  # type: ignore
         if connect:
             self.connect()
 
-    def connect(
-        self,
-        auth_methods: List = None,
-        silent: bool = False,
-    ):
-        """Authenticate with the SDK."""
+    def connect(self, auth_methods: List = None, silent: bool = False):
+        """
+        Authenticate to the Azure SDK.
+
+        Parameters
+        ----------
+        auth_methods : List, optional
+            list of preferred authentication methods to use, by default None
+        silent : bool, optional
+            Set true to prevent output during auth process, by default False
+
+        Raises
+        ------
+        CloudError
+            If no valid credentials are found or if subscription client can't be created
+
+        """
+        auth_methods = auth_methods or self.az_cloud_config.auth_methods
         self.credentials = az_connect(auth_methods=auth_methods, silent=silent)
         if not self.credentials:
             raise CloudError("Could not obtain credentials.")
         self._check_client("sub_client")
-        self.sub_client = SubscriptionClient(self.credentials.legacy)
+        if only_interactive_cred(self.credentials.modern) and not silent:
+            print("Check your default browser for interactive sign-in prompt.")
+
+        self.sub_client = SubscriptionClient(
+            credential=self.credentials.modern,
+            base_url=self.endpoints.resource_manager,
+            credential_scopes=[self.az_cloud_config.token_uri],
+        )
         if not self.sub_client:
             raise CloudError("Could not create a Subscription client.")
         self.connected = True
 
     def get_subscriptions(self) -> pd.DataFrame:
-        """Get details of all subscriptions within the tenant."""
+        """
+        Get details of all subscriptions within the tenant.
+
+        Returns
+        -------
+        pd.DataFrame
+            Details of the subscriptions present in the users tenant.
+
+        Raises
+        ------
+        MsticpyNotConnectedError
+            If .connect() has not been called
+
+        """
         if self.connected is False:
             raise MsticpyNotConnectedError(
                 "You need to connect to the service before using this function.",
@@ -181,6 +221,16 @@ class AzureData:
         sub_id : str
             The ID of the subscription to return details on.
 
+        Returns
+        -------
+        dict
+            Details on the selected subscription.
+
+        Raises
+        ------
+        MsticpyNotConnectedError
+            If .connect() has not been called.
+
         """
         if self.connected is False:
             raise MsticpyNotConnectedError(
@@ -207,7 +257,7 @@ class AzureData:
         self, sub_id: str, rgroup: str = None, get_props: bool = False
     ) -> pd.DataFrame:
         """
-        Return details on all resources in a subscription or Resoruce Group.
+        Return details on all resources in a subscription or Resource Group.
 
         Parameters
         ----------
@@ -221,7 +271,7 @@ class AzureData:
 
         Returns
         -------
-        resrouce_df: pd.DataFrame
+        pd.DataFrame
             A dataframe of resource details
 
         """
@@ -251,7 +301,7 @@ class AzureData:
 
         resource_items = []
 
-        # Get properites for each resource
+        # Get properties for each resource
         for resource in resources:
             if get_props:
                 if resource.type == "Microsoft.Compute/virtualMachines":
@@ -316,7 +366,7 @@ class AzureData:
 
         Returns
         -------
-        resource_deatils: dict
+        resource_details: dict
             The details of the requested resource
 
         """
@@ -434,7 +484,7 @@ class AzureData:
 
         self._check_client("resource_client", sub_id)  # type: ignore
 
-        # Normalise elements depending on user input type
+        # Normalize elements depending on user input type
         if resource_id is not None:
             try:
                 namespace = resource_id.split("/")[6]
@@ -723,9 +773,26 @@ class AzureData:
         if getattr(self, client_name) is None:
             client = _CLIENT_MAPPING[client_name]
             if sub_id is None:
-                setattr(self, client_name, client(self.credentials.modern))  # type: ignore
+                setattr(
+                    self,
+                    client_name,
+                    client(
+                        self.credentials.modern,  # type: ignore
+                        base_url=self.endpoints.resource_manager,
+                        credential_scopes=[self.az_cloud_config.token_uri],
+                    ),
+                )
             else:
-                setattr(self, client_name, client(self.credentials.modern, sub_id))  # type: ignore
+                setattr(
+                    self,
+                    client_name,
+                    client(
+                        self.credentials.modern,  # type: ignore
+                        sub_id,
+                        base_url=self.endpoints.resource_manager,
+                        credential_scopes=[self.az_cloud_config.token_uri],
+                    ),
+                )
 
         if getattr(self, client_name) is None:
             raise CloudError("Could not create client")
@@ -744,6 +811,23 @@ class AzureData:
         """
         client = _CLIENT_MAPPING[client_name]
         if sub_id is None:
-            setattr(self, client_name, client(self.credentials.legacy))  # type: ignore
+            setattr(
+                self,
+                client_name,
+                client(
+                    self.credentials.legacy,  # type: ignore
+                    base_url=self.endpoints.resource_manager,
+                    credential_scopes=[self.az_cloud_config.token_uri],
+                ),
+            )
         else:
-            setattr(self, client_name, client(self.credentials.legacy, sub_id))  # type: ignore
+            setattr(
+                self,
+                client_name,
+                client(
+                    self.credentials.legacy,  # type: ignore
+                    sub_id,
+                    base_url=self.endpoints.resource_manager,
+                    credential_scopes=[self.az_cloud_config.token_uri],
+                ),
+            )
