@@ -71,7 +71,7 @@ class RegisteredWidget(ABC):
     initialization parameters. If an instance of the same widget class is
     created with the same parameters, its previous value will be repopulated
     from the registry.
-    This is especially useful in notebooks where people accidently re-run
+    This is especially useful in notebooks where people accidentally re-run
     the same cell after entering values.
     """
 
@@ -538,6 +538,8 @@ class QueryTime(RegisteredWidget):
         )
         self._w_start_time_txt.value = self._query_start.isoformat(sep=" ")
         self._w_end_time_txt.value = self._query_end.isoformat(sep=" ")
+        self.before = abs(self._w_tm_range.value[0])
+        self.after = abs(self._w_tm_range.value[1])
 
     @property
     def start(self):
@@ -559,6 +561,11 @@ class QueryTime(RegisteredWidget):
         """Return the timespan as a TimeSpan object."""
         return TimeSpan(start=self.start, end=self.end)
 
+    @property
+    def value(self):
+        """Return the timespan as a TimeSpan object."""
+        return self.timespan
+
     def _ipython_display_(self):
         """Display in IPython."""
         self.display()
@@ -1007,7 +1014,7 @@ class GetText(RegisteredWidget):
     @property
     def value(self):
         """Get the current value of the key."""
-        return self._value.strip()
+        return self._value.strip() if self._value else None
 
     def display(self):
         """Display the interactive widgets."""
@@ -1476,8 +1483,10 @@ class Progress:
             orientation="horizontal",
         )
         self._done_label = widgets.Label(value="0%")
-        self._progress.visible = visible
-        self._done_label.visible = visible
+        if visible:
+            self.show()
+        else:
+            self.hide()
         self.layout = widgets.HBox([self._progress, self._done_label])
         self.display()
 
@@ -1529,13 +1538,16 @@ class Progress:
 
     def show(self):
         """Make the controls visible."""
-        self._progress.visible = True
-        self._done_label.visible = True
+        self._hide_show("visible")
 
     def hide(self):
         """Hide the controls."""
-        self._progress.visible = True
-        self._done_label.visible = True
+        self._hide_show("hidden")
+
+    def _hide_show(self, visibility):
+        vis_layout = widgets.Layout(visibility=visibility)
+        self._progress.layout = vis_layout
+        self._done_label.layout = vis_layout
 
     def display(self):
         """Display the control."""
@@ -1633,7 +1645,7 @@ class OptionButtons:
 
         def getvalue(change):
             """Button on_click handler."""
-            self.value = change.description.casefold()
+            self.value = change.description
             for btn in btns:
                 btn.on_click(getvalue, remove=True)
 
