@@ -43,6 +43,7 @@ from requests.exceptions import HTTPError
 from .._version import VERSION
 from ..common.exceptions import MsticpyUserConfigError
 from ..common.provider_settings import ProviderSettings, get_provider_settings
+from ..common.pkg_config import current_config_path
 from ..common.utility import export
 from ..datamodel.entities import GeoLocation, IpAddress
 
@@ -219,10 +220,7 @@ Alternatively, you can pass this to the IPStackLookup class when creating it:
         super().__init__()
 
         self.settings = _get_geoip_provider_settings("IPStack")
-        if api_key:
-            self._api_key = api_key
-        else:
-            self._api_key = self.settings.args.get("AuthKey")  # type: ignore
+        self._api_key = api_key or self.settings.args.get("AuthKey")
         if not self._api_key:
             raise MsticpyUserConfigError(
                 self._NO_API_KEY_MSSG,
@@ -360,18 +358,15 @@ Alternatively, you can pass this to the IPStackLookup class when creating it:
                 response = session.get(submit_url)
                 if response.status_code == 200:
                     ip_loc_results.append((response.json(), response.status_code))
+                elif response:
+                    try:
+                        ip_loc_results.append((response.json(), response.status_code))
+                        continue
+                    except JSONDecodeError:
+                        ip_loc_results.append((None, response.status_code))
                 else:
-                    if response:
-                        try:
-                            ip_loc_results.append(
-                                (response.json(), response.status_code)
-                            )
-                            continue
-                        except JSONDecodeError:
-                            ip_loc_results.append((None, response.status_code))
-                    else:
-                        print("Unknown response from IPStack request.")
-                        ip_loc_results.append((None, -1))
+                    print("Unknown response from IPStack request.")
+                    ip_loc_results.append((None, -1))
         return ip_loc_results
 
 
@@ -423,6 +418,7 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
         db_folder: Optional[str] = None,
         force_update: bool = False,
         auto_update: bool = True,
+        debug: bool = False,
     ):
         r"""
         Return new instance of GeoLiteLookup class.
@@ -448,12 +444,24 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
         auto_update: bool, optional
             Auto update can be set to true or false. depending on it,
             new download request will be initiated if age criteria is matched.
+        debug : bool, optional
+            Print additional debugging information, default is False.
 
         """
         super().__init__()
 
+        self._debug = debug
+        if self._debug:
+            dbg_api_key = (
+                "None" if api_key is None else api_key[:4] + "*" * (len(api_key) - 4)
+            )
+            self._pr_debug(f"__init__ params: api_key={dbg_api_key}")
+            self._pr_debug(f"    db_folder={db_folder}")
+            self._pr_debug(f"    force_update={force_update}")
+            self._pr_debug(f"    auto_update={auto_update}")
         self.settings = _get_geoip_provider_settings("GeoIPLite")
         self._api_key = api_key or self.settings.args.get("AuthKey")
+
         self._dbfolder = db_folder
         if self._dbfolder is None:
             self._dbfolder = self.settings.args.get("DBFolder", self._DB_HOME)
@@ -463,6 +471,19 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
         self._auto_update = auto_update
         self._check_and_update_db(self._dbfolder, self._force_update, self._auto_update)
         self._dbpath = self._get_geoip_dbpath(self._dbfolder)
+        if self._debug:
+            dbg_api_key = (
+                "None"
+                if self._api_key is None
+                else self._api_key[:4] + "*" * (len(self._api_key) - 4)
+            )
+            self._pr_debug(f"__init__ values (inc settings): api_key={dbg_api_key}")
+            self._pr_debug(f"    db_folder={self._dbfolder}")
+            self._pr_debug(f"    force_update={self._force_update}")
+            self._pr_debug(f"    auto_update={self._auto_update}")
+            self._pr_debug(f"    dbpath={self._dbpath}")
+            self._pr_debug(f"Using config file: {current_config_path()}")
+
         if not self._dbpath:
             raise MsticpyUserConfigError(
                 "No usable GeoIP Database could be found.",
@@ -472,7 +493,7 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
                 ),
                 (
                     "If you are using a custom DBFolder setting in your config, "
-                    + "check that this is a valid path."
+                    + f"check that this is a valid path: {self._dbfolder}."
                 ),
                 (
                     "If you edit your msticpyconfig to change this setting run the "
@@ -526,7 +547,8 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
         """
         geoip_db_path = self._get_geoip_dbpath(db_folder)
         url = self._MAXMIND_DOWNLOAD.format(license_key=self._api_key)
-
+        self._pr_debug(f"Checking geoip DB {geoip_db_path}")
+        self._pr_debug(f"Download URL is {self._MAXMIND_DOWNLOAD}")
         if geoip_db_path is None:
             print(
                 "No local Maxmind City Database found. ",
@@ -548,7 +570,8 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
                     f"Attempting to download new database to {db_folder}",
                 )
                 if not self._download_and_extract_archive(url, db_folder):
-                    warnings.warn("DB download failed")
+                    self._pr_debug("DB download failed")
+                    self._geolite_warn("DB download failed")
                     db_updated = False
             elif force_update:
                 print(
@@ -556,10 +579,12 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
                     f"Attempting to download new database to {db_folder}",
                 )
                 if not self._download_and_extract_archive(url, db_folder):
-                    warnings.warn("DB download failed")
+                    self._pr_debug("DB download failed")
+                    self._geolite_warn("DB download failed")
                     db_updated = False
             if not db_updated:
-                warnings.warn(
+                self._pr_debug("Continuing with cached database.")
+                self._geolite_warn(
                     "Continuing with cached database. Results may inaccurate."
                 )
 
@@ -601,8 +626,7 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
         db_archive_path = Path(db_folder).joinpath(
             self._DB_ARCHIVE.format(rand=rand_int)
         )
-        db_file_path = Path(db_folder).joinpath(self._DB_FILE)
-
+        self._pr_debug(f"Downloading GeoLite DB: {db_archive_path}")
         try:
             # wait a small rand amount of time in case multiple procs try
             # to download simultaneously
@@ -618,44 +642,62 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
                     for chunk in response.iter_content(chunk_size=10000):
                         file_hdl.write(chunk)
                         file_hdl.flush()
+            self._pr_debug(f"Downloaded GeoLite DB: {db_archive_path}")
         except HTTPError as http_err:
-            warnings.warn(
+            self._pr_debug(
+                f"HTTP error occurred trying to download GeoLite DB: {http_err}"
+            )
+            self._geolite_warn(
                 f"HTTP error occurred trying to download GeoLite DB: {http_err}"
             )
         # pylint: disable=broad-except
         except Exception as err:
-            warnings.warn(f"Other error occurred trying to download GeoLite DB: {err}")
+            self._pr_debug(f"Other error occurred trying to download GeoLite DB: {err}")
+            self._geolite_warn(
+                f"Other error occurred trying to download GeoLite DB: {err}"
+            )
         # pylint: enable=broad-except
         else:
             try:
                 self._extract_to_folder(db_archive_path, db_folder)
-                print("Extraction complete. Local Maxmind city DB:", f"{db_file_path}")
+                print(
+                    "Extraction complete. Local Maxmind city DB:", f"{db_archive_path}"
+                )
                 return True
             except PermissionError as err:
-                warnings.warn(
-                    f"Cannot overwrite GeoIP DB file: {db_file_path}."
+                self._pr_debug(
+                    f"Error writing GeoIP DB file: {db_archive_path} - {err}"
+                )
+                self._geolite_warn(
+                    f"Cannot overwrite GeoIP DB file: {db_archive_path}."
                     + " The file may be in use or you do not have"
                     + f" permission to overwrite.\n - {err}"
                 )
             except Exception as err:  # pylint: disable=broad-except
                 # There are several exception types that might come from
                 # unpacking a tar.gz
-                warnings.warn(f"Error writing GeoIP DB file: {db_file_path} - {err}")
+                self._pr_debug(
+                    f"Error writing GeoIP DB file: {db_archive_path} - {err}"
+                )
+                self._geolite_warn(
+                    f"Error writing GeoIP DB file: {db_archive_path} - {err}"
+                )
         finally:
             if db_archive_path.is_file():
+                self._pr_debug(f"Removing temp file {db_archive_path}")
                 db_archive_path.unlink()
         return False
 
     # pylint: enable=too-many-branches
 
-    @staticmethod
-    def _extract_to_folder(db_archive_path, db_folder):
+    def _extract_to_folder(self, db_archive_path, db_folder):
+        self._pr_debug(f"Extracting tarfile {db_archive_path}")
         with tarfile.open(db_archive_path) as tar_archive:
             for member in tar_archive.getmembers():
                 if not member.isreg():
                     continue
                 # Will skip the dirs to extract only file objects
-
+                self._pr_debug(f"extracting {member} to {db_folder}")
                 tar_archive.extract(member, db_folder)
                 # The files are extract to a subfolder (with a date in the name)
                 # We want to move these into the main folder above this.
@@ -664,6 +706,7 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
                     curr_file = Path(db_folder).joinpath(member.name)
                     extr_folder = Path(db_folder).joinpath(member.name).parent
                     curr_file.replace(Path(db_folder).joinpath(targetname))
+                    self._pr_debug(f"Moving to {Path(db_folder).joinpath(targetname)}")
                     # if the folder is empty, remove it
                     if not list(extr_folder.glob("*")):
                         extr_folder.rmdir()
@@ -774,6 +817,18 @@ Alternatively, you can pass this to the GeoLiteLookup class when creating it:
         ip_entity.Location = geo_entity
         return ip_entity
 
+    def _pr_debug(self, *args):
+        """Print out debug info."""
+        if self._debug:
+            print(*args)
+
+    @staticmethod
+    def _geolite_warn(mssg):
+        warnings.warn(
+            f"GeoIpLookup: {mssg}",
+            UserWarning,
+        )
+
 
 def _get_geoip_provider_settings(provider_name: str) -> ProviderSettings:
     """
