@@ -14,6 +14,7 @@ requests per minute for the account type that you have.
 """
 import abc
 from abc import ABC
+import collections
 import math  # noqa
 import pprint
 import re
@@ -45,11 +46,15 @@ SanitizedObservable = namedtuple("SanitizedObservable", ["observable", "status"]
 class TISeverity(Enum):
     """Threat intelligence report severity."""
 
+    # pylint: disable=invalid-name
     unknown = -1
     information = 0
     warning = 1
     high = 2
 
+    # pylint: enable=invalid-name
+
+    # pylint: disable=unsupported-membership-test, no-member
     @classmethod
     def parse(cls, value) -> "TISeverity":
         """
@@ -70,11 +75,14 @@ class TISeverity(Enum):
             return value
         if isinstance(value, str) and value.lower() in cls.__members__:
             return cls[value.lower()]
-        if isinstance(value, int):
-            if value in [v.value for v in cls.__members__.values()]:
-                return cls(value)
+        if isinstance(value, int) and value in [
+            v.value for v in cls.__members__.values()
+        ]:
+            return cls(value)
         return TISeverity.unknown
 
+    # pylint: enable=unsupported-membership-test, no-member
+
     # pylint: disable=comparison-with-callable
     def __eq__(self, other) -> bool:
         """
@@ -190,7 +198,7 @@ class LookupResult:
         """Return a dictionary that maps fields to DF Names."""
         col_mapping = {}
         for name in attr.fields_dict(cls):
-            out_name = "".join([part.capitalize() for part in name.split("_")])
+            out_name = "".join(part.capitalize() for part in name.split("_"))
             col_mapping[name] = out_name
         return col_mapping
 
@@ -198,7 +206,7 @@ class LookupResult:
 # pylint: enable=too-many-instance-attributes
 
 
-# pylint: disable=too-few-public-methods
+# pylint: disable=too-few-public-methods, invalid-name
 class TILookupStatus(Enum):
     """Threat intelligence lookup status."""
 
@@ -209,7 +217,7 @@ class TILookupStatus(Enum):
     other = 10
 
 
-# pylint: enable=too-few-public-methods
+# pylint: enable=too-few-public-methods, invalid-name
 
 
 _IOC_EXTRACT = IoCExtract()
@@ -338,6 +346,19 @@ class TIProvider(ABC):
         """
         return [ioc.name for ioc in self._supported_types]
 
+    @property
+    def ioc_query_defs(self) -> Dict[str, Any]:
+        """
+        Return current dictionary of IoC query/request definitions.
+
+        Returns
+        -------
+        Dict[str, Any]
+            IoC query/requist definitions keyed by IoCType
+
+        """
+        return self._IOC_QUERIES
+
     @classmethod
     def is_known_type(cls, ioc_type: str) -> bool:
         """
@@ -433,7 +454,7 @@ class TIProvider(ABC):
         result = LookupResult(
             ioc=ioc,
             safe_ioc=ioc,
-            ioc_type=ioc_type if ioc_type else self.resolve_ioc_type(ioc),
+            ioc_type=ioc_type or self.resolve_ioc_type(ioc),
             query_subtype=query_subtype,
             result=False,
             details="",
@@ -700,7 +721,8 @@ def generate_items(
 
     """
     del obs_col, ioc_type_col
-    if isinstance(data, Iterable):
+
+    if isinstance(data, collections.abc.Iterable):
         for item in data:
             yield item, TIProvider.resolve_ioc_type(item)
     else:
